#include "parse_util.h"
#include "parser.tab.h"

int char_num = 1;
int line_num = 1;
int s_char_num = 1;
int s_line_num = 1;

extern int yyparse();

void yyerror (const char *s) {
  cout << "Error! " << s << " at line " << s_line_num << " character " << s_char_num << endl;
}


StmtsNode::StmtsNode(GeneralNode *e, GeneralNode *nxt) {
  stmt = e;
  next = nxt;
}

void StmtsNode::print(ostream& os = cout) {
  if (stmt) {
    stmt->print(os);
    os << ";" << "\n";
    if (next)
      next->print(os);
  }
}

BinaryNode::BinaryNode(int op, GeneralNode *arg1, GeneralNode *arg2) {
  theop = op;
  leftarg = arg1;
  rightarg = arg2;
}

void BinaryNode::print(ostream& os = cout) {
  leftarg->print(os);
  switch (theop) {
    case T_TIMES: os << " * "; break;
    case T_PLUS: os << " + "; break;
    default: cerr << "Problem!" << endl; break;
  }
  rightarg->print(os);
}

UnaryNode::UnaryNode(int op, GeneralNode *thearg) {
  theop = op;
  arg = thearg;
}

void UnaryNode::print(ostream& os = cout) {
  if (theop == T_LPAREN) {
    os << "( ";
    arg->print(os);
    os << " )";
  }
  else
    cerr << "Problem!" << endl;
}

IntNode::IntNode(int ival) {
  ivalue = ival;
}

void IntNode::print(ostream& os = cout) {
  os << ivalue;
}

GeneralNode *parse_root = 0;

GeneralNode *do_parse(const char* filename) {
  if ((yyin = fopen(filename,"r")) == NULL) {
    cout << "Error!  Unable to open file " << filename << endl;
    return 0;
  }

  if (yyparse()) {
    return 0;
  }
  else {
    return parse_root;
  }
}
